/*-
 * %%----------------------------------------------------------------------------------------------
 * Archive ouverte UNIGE - AoU Access - PublicationDownloadJmsListener.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2022 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.aou.service;

import java.util.NoSuchElementException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Service;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.controller.AccessController;
import ch.unige.aou.message.PublicationDownloadMessage;
import ch.unige.aou.model.access.PublicationDownload;

@Service
@ConditionalOnBean(AccessController.class)
public class PublicationDownloadJmsListener extends MessageProcessor<PublicationDownloadMessage> {

  private static final Logger log = LoggerFactory.getLogger(PublicationDownloadJmsListener.class);

  private final PublicationDownloadService archiveDownloadService;
  private final PublicationDownloadProcessingService publicationDownloadProcessingService;

  public PublicationDownloadJmsListener(AouProperties aouProperties, PublicationDownloadService archiveDownloadService,
          PublicationDownloadProcessingService publicationDownloadProcessingService) {
    super(aouProperties);
    this.archiveDownloadService = archiveDownloadService;
    this.publicationDownloadProcessingService = publicationDownloadProcessingService;
  }

  @Override
  public void processMessage(PublicationDownloadMessage publicationDownloadMessage) {
    log.info("Reading message {}", publicationDownloadMessage);
    try {
      PublicationDownload publicationDownload = this.archiveDownloadService.findOne(publicationDownloadMessage.getResId());
      this.publicationDownloadProcessingService.processArchiveDownloadUntilReady(publicationDownload);
      log.info("PublicationDownload '{}' processed", publicationDownload.getResId());
    } catch (NoSuchElementException E) {
      log.error("Cannot find archive ({})", publicationDownloadMessage.getResId());
    }
  }

  @JmsListener(destination = "${aou.queue.publicationDownload}")
  @Override
  public void receiveMessage(PublicationDownloadMessage publicationDownloadMessage) {
    this.sendForParallelProcessing(publicationDownloadMessage);
  }
}
